/*
Danger from the Deep - Open source submarine simulation
Copyright (C) 2003-2006  Thorsten Jordan, Luis Barrancos and others.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// sound
// subsim (C)+(W) Markus Petermann and Thorsten Jordan. SEE LICENSE

#ifndef _SOUND_H_
#define _SOUND_H_

#include "angle.h"
#include "sea_object.h"
#include <SDL_mixer.h>

///\brief Representation of a sound effect.
class sound
{
public:
	sound(const std::string& filename);
	~sound();
	void play(float volume = 1.0f) const;	
	void play(const sea_object* player, const sea_object* noise_source, const bool loop_forever = false);
	void play_fade(const sea_object* player, const sea_object* noise_source, const bool loop_forever = false);
	void stop();
	void fade_out();

	static bool use_sound;	// set this to enable/disable sound

	static void init();
	static void deinit();
	
	void pause();
	void resume();
	void pause_all();
	void resume_all();

protected:
	sound();
	sound(const sound& );
	sound& operator= (const sound& );

	Mix_Chunk* mixchunk;
	int channel_num;
	
private:
	void _play(const sea_object* player, const sea_object* noise_source, const bool loop_forever, const bool fade);
};

#endif /* _SOUND_H_ */
